@@ -262,7 +262,7 @@ gtk_button_set_image(GTK_BUTTON(priv->btn_stop), gtk_image_new_from_stock(GTK_ST
 
 priv->btn_delete=gtk_button_new();
 gtk_button_set_image(GTK_BUTTON(priv->btn_delete), gtk_image_new_from_stock(GTK_STOCK_DELETE, GTK_ICON_SIZE_BUTTON));
-priv->allow_delete=FALSE;
+priv->allow_delete=TRUE;
 
 #if GTK_CHECK_VERSION(2,10,0)
 gtk_button_set_image_position(GTK_BUTTON(priv->btn_record), GTK_POS_TOP);
@@ -481,14 +481,8 @@ if (!model)
     return;
 gtk_tree_model_get(model, &iter, NOTE_ID, &file, -1);
 
-g_debug("File %s activated", file);
-
-#if 1
+g_debug("dicto-ui: File %s activated", file);
 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->btn_play), TRUE);
-#else
-if (!dicto_play(priv->dicto, file))
-	dicto_widget_update_view_mode(priv);
-#endif
 
 g_free(file);
 }
@@ -593,9 +587,10 @@ switch (prop_id) {
 		g_debug("Set dicto object");
 		d=g_value_get_object(value);
 		g_return_if_fail(IS_DICTO(d));
+		g_object_ref(d);
 
+		/* Disconnect previous Dicto */
 		if (priv->dicto) {
-			/* XXX: disconnect signals */
 			dicto_stop(priv->dicto);
 			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_position_cb), dw);
 			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_seek_cb), dw);
@@ -610,10 +605,8 @@ switch (prop_id) {
 			g_signal_handlers_disconnect_by_func(G_OBJECT(priv->dicto), G_CALLBACK(dicto_widget_error_cb), dw);
 			g_object_unref(priv->dicto);
 		}
-		g_object_ref(d);
+
 		priv->dicto=d;
-		dicto_widget_updatelist_cb(priv->dicto, dw);
-		dicto_widget_update_view_mode(priv);
 		g_signal_connect(G_OBJECT(priv->dicto), "position", G_CALLBACK(dicto_widget_position_cb), dw);
 		g_signal_connect(G_OBJECT(priv->dicto), "seeked", G_CALLBACK(dicto_widget_seek_cb), dw);
 		g_signal_connect(G_OBJECT(priv->dicto), "playing", G_CALLBACK(dicto_widget_update_view_mode_cb), dw);
@@ -625,6 +618,8 @@ switch (prop_id) {
 		g_signal_connect(G_OBJECT(priv->dicto), "ready", G_CALLBACK(dicto_widget_ready_cb), dw);
 		g_signal_connect(G_OBJECT(priv->dicto), "level", G_CALLBACK(dicto_widget_level_cb), dw);
 		g_signal_connect(G_OBJECT(priv->dicto), "error", G_CALLBACK(dicto_widget_error_cb), dw);
+		dicto_refresh_list(priv->dicto);
+		dicto_widget_update_view_mode(priv);
 	break;
 	case PROP_ENABLE_DELETE:
 		priv->allow_delete=g_value_get_boolean(value);
@@ -702,6 +697,7 @@ if (priv->dicto)
 g_debug("Mode: %d", mode);
 
 switch (mode) {
+	case DICTO_STATE_UNKNOWN:
 	case DICTO_STATE_ERROR:
 		/* XXX: What should we do ?*/
 	case DICTO_STATE_STOPPED:
@@ -730,6 +726,7 @@ switch (mode) {
 		gtk_widget_set_sensitive(priv->btn_stop, TRUE);
 		gtk_widget_set_sensitive(priv->btn_delete, FALSE);
 		gtk_range_set_update_policy(GTK_RANGE(priv->position), GTK_UPDATE_CONTINUOUS);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->btn_play), FALSE);
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(priv->level), 0.0);
 	break;
 	case DICTO_STATE_READY:
@@ -827,7 +824,7 @@ DictoWidgetPriv *priv;
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
 g_return_val_if_fail(priv, TRUE);
 
-g_debug("Got error from dicto");
+dicto_widget_update_view_mode(priv);
 return FALSE;
 }
 
@@ -953,12 +950,15 @@ return TRUE;
 gboolean
 dicto_widget_delete_query_dialog(GtkWindow *w, const gchar *file)
 {
+gchar *msg;
 GtkWidget *nh;
 gint r;
 
-nh=hildon_note_new_confirmation(w, "Delete ?");
+msg=g_strdup_printf("Delete note: '%s' ?", file);
+nh=hildon_note_new_confirmation(w, msg);
 r=gtk_dialog_run(GTK_DIALOG(nh));
 gtk_widget_destroy(nh);
+g_free(msg);
 
 return (r==GTK_RESPONSE_OK);
 }
