@@ -39,6 +39,7 @@
 #endif
 
 #include <string.h>
+#include <unistd.h>
 #include <math.h>
 
 #include <glib.h>
@@ -98,9 +99,13 @@ struct _dicto_pipeline {
 	GstElement *sink;
 	GstElement *encoder;
 	GstElement *level;
+	GstBus *bus;
+	GstTagList *tags;
+	GstTagSetter *ts;
 	GstCaps	*caps;
 	gboolean active;
 	gboolean rec;
+	gboolean doseek;
 	gdouble position;
 	gdouble length;
 	GObject *owner;
@@ -155,13 +160,17 @@ G_OBJECT_CLASS(dicto_parent_class)->dispose(object);
 static void
 dicto_pipeline_free(dicto_pipeline *np)
 {
-if (np->pipeline) {
-	gst_element_set_state(np->pipeline, GST_STATE_NULL);
-	gst_object_unref(np->pipeline);
-	np->pipeline=NULL;
-	g_object_unref(np->owner);
-	np->owner=NULL;
-}
+if (!np->pipeline)
+	return;
+gst_bus_set_flushing(np->bus, TRUE);
+gst_object_unref(np->bus);
+gst_element_set_state(np->pipeline, GST_STATE_NULL);
+
+gst_object_unref(np->pipeline);
+np->pipeline=NULL;
+
+g_object_unref(np->owner);
+np->owner=NULL;
 }
 
 static void
@@ -276,35 +285,45 @@ switch (prop_id) {
 }
 }
 
-static void
-dicto_constructed(GObject *object)
+static GObject*
+dicto_constructor(GType type, guint n_construct_properties, GObjectConstructParam *construct_properties)
 {
-guint i=0;
-Dicto *d=DICTO(object);
-DictoPrivate *p=GET_PRIVATE(d);
-DictoFormat *f;
+Dicto *d;
+DictoPrivate *p;
+GObject *object;
 
-p->formats=g_hash_table_new(g_direct_hash, g_direct_equal);
+object=G_OBJECT_CLASS(dicto_parent_class)->constructor(type, n_construct_properties, construct_properties);
 
-while (dicto_formats[i].channels>0) {
-	f=&dicto_formats[i];
-	dicto_recording_format_add(d, f->id, f);
-	i++;
-}
+d=DICTO(object);
+p=GET_PRIVATE(d);
+
+if (!dicto_create_play_pipeline(d, NULL))
+	g_warning("Failed to create initial playback pipeline");
+if (!dicto_create_record_pipeline(d, p->format, NULL))
+	g_warning("Failed to create initial recording pipeline");
 
-p->rms=0;
-p->timefmt=g_strdup(DEF_TIME_FORMAT);
-p->pos_int=POS_INT_DEFAULT;
-p->tags=NULL;
- 
-dicto_create_play_pipeline(d, NULL);
-dicto_create_record_pipeline(d, p->format, NULL);
 dicto_notes_load_list(d, FALSE);
+
+return object;
 }
 
 static void 
 dicto_init(Dicto *d)
 {
+guint i=0;
+DictoFormat *f;
+DictoPrivate *p=GET_PRIVATE(d);
+
+p->formats=g_hash_table_new(g_direct_hash, g_direct_equal);
+while (dicto_formats[i].channels>0) {
+	f=&dicto_formats[i];
+	dicto_recording_format_add(d, f->id, f);
+	i++;
+}
+p->rms=0.0;
+p->timefmt=g_strdup(DEF_TIME_FORMAT);
+p->pos_int=POS_INT_DEFAULT;
+p->tags=NULL;
 }
 
 static void
@@ -317,7 +336,7 @@ object_class->dispose=dicto_dispose;
 object_class->finalize=dicto_finalize;
 object_class->set_property=dicto_set_property;
 object_class->get_property=dicto_get_property;
-object_class->constructed=dicto_constructed;
+object_class->constructor=dicto_constructor;
 
 g_type_class_add_private(object_class, sizeof (DictoPrivate));
 
@@ -512,7 +531,7 @@ return TRUE;
  * Remove a recording format.
  *
  */
-static gboolean
+gboolean
 dicto_recording_format_remove(Dicto *d, guint f_id)
 {
 DictoPrivate *p=GET_PRIVATE(d);
@@ -522,28 +541,38 @@ return g_hash_table_remove(p->formats, GINT_TO_POINTER(f_id));
 
 /**
  * dicto_set_basedir:
+ * @d: A #Dicto object.
+ * @basedir: An absolute UNIX path to notes, it will be created if it does not exists.
  * 
  * Set base directory to use to for recordings.
- * @d: A #Dicto object.
- * @basedir: A UNIX path to notes, it will be created if it does not exists.
  *
+ * Returns: TRUE if new basedir was set. FALSE if basedir is not a directory or if creation failed.
  */
 gboolean
 dicto_set_basedir(Dicto *d, const gchar *basedir)
 {
+gchar *tmp;
 DictoPrivate *p=GET_PRIVATE(d);
 
 g_return_val_if_fail(d, FALSE);
 g_return_val_if_fail(basedir, FALSE);
-g_return_val_if_fail(g_path_is_absolute(basedir), FALSE);
+
+if (g_path_is_absolute(basedir)==FALSE)
+	return FALSE;
+
+if (g_file_test(basedir, G_FILE_TEST_IS_REGULAR)==TRUE)
+	return FALSE;
+
+tmp=g_strdup(basedir);
+if (g_mkdir_with_parents(tmp, 0775)==-1) {
+	g_warning("Failed to create basedirectory: %s", tmp);
+	g_free(tmp);
+	return FALSE;
+}
 
 if (p->basedir)
 	g_free(p->basedir);
-p->basedir=g_strdup(basedir);
-if (g_mkdir_with_parents(p->basedir, 0775)==-1) {
-	g_warning("Failed to create basedirectory: %s", p->basedir);
-	return FALSE;
-}
+p->basedir=tmp;
 
 g_debug("Basedir: %s", p->basedir);
 
@@ -581,6 +610,8 @@ static void
 dicto_load_tag(const GstTagList *list, const gchar *tag, gpointer data)
 {
 gchar *v;
+guint i;
+gboolean b;
 GType t;
 Dicto *d=(Dicto *)data;
 DictoPrivate *p=GET_PRIVATE(d);
@@ -597,7 +628,17 @@ switch (t) {
 		g_hash_table_insert(p->tags, g_strdup(tag), v);
 	}
 	break;
-	default:;
+	case G_TYPE_BOOLEAN:
+		if (gst_tag_list_get_boolean(list, tag, &b))
+			g_debug("Tag: %s = %d", tag, b);
+	break;
+	case G_TYPE_UINT:
+		if (gst_tag_list_get_uint(list, tag, &i))
+			g_debug("Tag: %s = %d", tag, i);
+	break;
+	default:
+		g_debug("Tag-type: %d: %s", (guint)t, g_type_name(t));
+	break;
 }
 
 }
@@ -609,9 +650,9 @@ gint channels, i;
 const GValue *list;
 DictoPrivate *p=GET_PRIVATE(d);
 
-*rms_dB=0;
-*peak_dB=0;
-*decay_dB=0;
+*rms_dB=0.0;
+*peak_dB=0.0;
+*decay_dB=0.0;
 
 list=gst_structure_get_value(s, "rms");
 if (!G_IS_VALUE(list))
@@ -633,6 +674,16 @@ p->rms=*rms_dB/channels;
 return TRUE;
 }
 
+static void
+dicto_pipeline_stop(DictoPrivate *p, dicto_pipeline *np)
+{
+gst_element_set_state(np->pipeline, GST_STATE_NULL);
+dicto_position_update(DICTO(np->owner), FALSE);
+np->active=FALSE;
+p->rms=0.0;
+p->state=DICTO_STATE_STOPPED;
+}
+
 static gboolean
 dicto_bus_cb(GstBus *bus, GstMessage *msg, gpointer data)
 {
@@ -649,19 +700,19 @@ DictoPrivate *p=GET_PRIVATE(d);
 switch (GST_MESSAGE_TYPE(msg)) {
 	case GST_MESSAGE_EOS:
 		if (np->rec) {
-			np->active=FALSE;
-			gst_element_set_state(np->pipeline, GST_STATE_NULL);
-			dicto_position_update(DICTO(np->owner), FALSE);
-			p->state=DICTO_STATE_STOPPED;
+			g_debug("dicto: EOS, stopping recording");
+			dicto_pipeline_stop(p, np);
 			g_signal_emit(G_OBJECT(d), signals[SIGNAL_STOPPED_RECORD], 0, NULL);
 		} else {
+			g_debug("dicto: EOS, pausing playback");
 			if (dicto_pause(d))
 				dicto_seek(d, 0.0);
+			dicto_position_cb(np->owner);
 		}
 	break;
 	case GST_MESSAGE_ERROR:
 		gst_message_parse_error(msg, &err, &debug);
-		g_warning("GstBusError: %s (%s)", err->message, debug);
+		g_warning("dicto: GstBusError: %s (%s)", err->message, debug);
 		g_free(debug);
 		np->active=FALSE;
 		dicto_position_update(DICTO(np->owner), FALSE);
@@ -711,20 +762,51 @@ switch (GST_MESSAGE_TYPE(msg)) {
 return TRUE;
 }
 
+/**
+ * dicto_new_wav_pad_cb:
+ *
+ * New pad handler for wavparse.
+ */
 static gboolean
-dicto_new_pad_cb(GstElement *wavparse, GstPad *new_pad, gpointer data)
+dicto_new_wav_pad_cb(GstElement *wavparse, GstPad *pad, gpointer data)
 {
 GstElement *sink=(GstElement *)data;
 
+g_debug("new-wav-pad");
 if (!gst_element_link(wavparse, sink)) {
 	g_warning("Failed to link wavparse to sink.");
 }
-
 gst_element_sync_state_with_parent(sink);
 
 return FALSE;
 }
 
+/**
+ * dicto_new_pad_cb:
+ *
+ * New pad handler for decodebin
+ */
+static void
+dicto_new_pad_cb(GstElement *decodebin, GstPad *pad, gboolean tmpdepre, gpointer data)
+{
+GstCaps *caps;
+GstStructure *str;
+GstElement *sink=(GstElement *)data;
+
+g_debug("new-pad");
+caps=gst_pad_get_caps(pad);
+str=gst_caps_get_structure(caps, 0);
+if (!g_strrstr(gst_structure_get_name(str), "audio")) {
+	g_debug("Not audio src");
+	gst_caps_unref(caps);
+	return;
+}
+gst_caps_unref(caps);
+
+if (!gst_element_link(decodebin, sink))
+	g_warning("Failed to link decodebin to sink.");
+}
+
 static GstCaps *
 dicto_get_src_caps(const gchar *type, gint depth, gint rate, gint ch)
 {
@@ -760,9 +842,8 @@ static gboolean
 dicto_create_record_pipeline(Dicto *d, DictoFormats nf, GError **error)
 {
 GError *err=NULL;
-GstBus *bus;
 GstCaps *caps;
-GstElement *enext, *ac;
+GstElement *enext, *ac, *tags;
 GstElement *pipeline, *encoder, *muxer, *filter, *queue, *sink, *src, *level;
 DictoFormat *f;
 DictoPrivate *p=GET_PRIVATE(d);
@@ -770,19 +851,18 @@ dicto_pipeline *np=&p->record;
 
 g_return_val_if_fail(p->gst_source, FALSE);
 
-if (p->record.active==TRUE)
+if (p->record.active==TRUE) {
+	g_warning("Can't change pipeline when active");
 	return FALSE;
+}
 
-if (p->play.active==TRUE)
-	return FALSE;
+f=&dicto_formats[nf];
+g_return_val_if_fail(f->id==nf, FALSE);
 
 encoder=NULL;
 muxer=NULL;
 filter=NULL;
 
-f=&dicto_formats[nf];
-g_return_val_if_fail(f->id==nf, FALSE);
-
 if (f->muxer) {
 	muxer=gst_element_factory_make(f->muxer, "muxer");
 	g_return_val_if_fail(muxer, FALSE);
@@ -900,9 +980,11 @@ if (!gst_element_link(enext, sink)) {
 }
 g_debug("(muxer)->sink");
 
-bus=gst_pipeline_get_bus(GST_PIPELINE(pipeline));
-gst_bus_add_watch(bus, dicto_bus_cb, np);
-gst_object_unref(bus);
+tags=gst_bin_get_by_interface(GST_BIN(pipeline), GST_TYPE_TAG_SETTER);
+np->ts=tags ? GST_TAG_SETTER(tags) : NULL;
+
+np->bus=gst_pipeline_get_bus(GST_PIPELINE(pipeline));
+gst_bus_add_watch(np->bus, dicto_bus_cb, np);
 
 /* Free the old pipeline */
 dicto_pipeline_free(np);
@@ -942,7 +1024,6 @@ return FALSE;
 static gboolean
 dicto_create_play_playbin_pipeline(Dicto *d)
 {
-GstBus *bus;
 DictoPrivate *p=GET_PRIVATE(d);
 dicto_pipeline *np=&p->play;
 
@@ -956,9 +1037,8 @@ if (!np->sink)
 
 g_object_set(G_OBJECT(np->pipeline), "audio-sink", np->sink, NULL);
 
-bus=gst_pipeline_get_bus(GST_PIPELINE(np->pipeline));
-gst_bus_add_watch(bus, dicto_bus_cb, np);
-gst_object_unref(bus);
+np->bus=gst_pipeline_get_bus(GST_PIPELINE(np->pipeline));
+gst_bus_add_watch(np->bus, dicto_bus_cb, np);
 np->rec=FALSE;
 np->active=FALSE;
 
@@ -972,9 +1052,8 @@ return FALSE;
 static gboolean
 dicto_create_play_pipeline(Dicto *d, GError **error)
 {
-GstBus *bus;
 DictoPrivate *p=GET_PRIVATE(d);
-GstElement *bsink, *filter, *queue;
+GstElement *bsink, *filter, *queue, *ac;
 dicto_pipeline *np=&p->play;
 
 g_return_val_if_fail(p->gst_sink, FALSE);
@@ -1005,12 +1084,17 @@ if (!queue)
 	goto error_play;
 g_debug("queue");
 
+ac=gst_element_factory_make("audioconvert", "ac");
+if (!ac)
+	goto error_play;
+g_debug("ac");
+
 np->sink=gst_element_factory_make(p->gst_sink, "asink");
 if (!np->sink)
 	goto error_play;
 g_debug("sink");
 
-gst_bin_add_many(GST_BIN(np->pipeline), np->src, queue, filter, np->sink, NULL);
+gst_bin_add_many(GST_BIN(np->pipeline), np->src, queue, filter, ac, np->sink, NULL);
 
 if (!gst_element_link(np->src, queue))
 	goto error_play;
@@ -1032,17 +1116,25 @@ if (!np->level) {
 	bsink=np->level;
 }
 
-g_signal_connect(filter, "pad_added", G_CALLBACK(dicto_new_pad_cb), bsink);
+if (!gst_element_link(ac, bsink))
+	goto error_play;
+g_debug("ac->sink");
 
-bus=gst_pipeline_get_bus(GST_PIPELINE(np->pipeline));
-gst_bus_add_watch(bus, dicto_bus_cb, np);
-gst_object_unref(bus);
+if (p->has_decodebin) {
+	g_signal_connect(filter, "new-decoded-pad", G_CALLBACK(dicto_new_pad_cb), ac);
+} else {
+	g_signal_connect(filter, "pad_added", G_CALLBACK(dicto_new_wav_pad_cb), ac);
+}
+
+np->bus=gst_pipeline_get_bus(GST_PIPELINE(np->pipeline));
+gst_bus_add_watch(np->bus, dicto_bus_cb, np);
 
 g_object_ref(d);
 np->owner=G_OBJECT(d);
 
 np->rec=FALSE;
 np->active=FALSE;
+np->doseek=FALSE;
 
 return TRUE;
 
@@ -1054,6 +1146,8 @@ return FALSE;
 }
 
 /**
+ * dicto_set_filename:
+ *
  * Set the given elements (filesrc or filesink) file location
  */
 static gboolean
@@ -1079,6 +1173,10 @@ if (g_path_is_absolute(file))
 else
 	tmp=g_build_filename(p->basedir, file, NULL);
 
+/* If string allocation and copy failed, then error out */
+if (tmp == NULL)
+	return FALSE;
+
 /* Is it already set, if so skip */
 g_object_get(G_OBJECT(e), "location", &ecf, NULL);
 if (p->cfile && ecf && strcmp(tmp, p->cfile)==0 && strcmp(tmp, ecf)==0) {
@@ -1100,7 +1198,9 @@ return TRUE;
 }
 
 /**
- * 
+ * dicto_notes_load_list:
+ *
+ *
  */
 static gboolean
 dicto_notes_load_list(Dicto *d, gboolean s)
@@ -1114,10 +1214,15 @@ g_return_val_if_fail(d, FALSE);
 if (!p->basedir)
 	return FALSE;
 
-if (!p->notes)
+if (!p->notes) {
 	p->notes=g_hash_table_new(g_str_hash, g_str_equal);
-else
+} else {
+#if (GLIB_CHECK_VERSION (2, 12, 0))
 	g_hash_table_remove_all(p->notes);
+#else
+	g_hash_table_foreach_remove(p->notes, gtk_true, NULL);
+#endif
+}
 
 ddir=g_dir_open(p->basedir, 0, NULL);
 if (!ddir)
@@ -1190,6 +1295,7 @@ return FALSE;
 static gboolean
 dicto_seek_pipeline(dicto_pipeline *p)
 {
+p->doseek=FALSE;
 return gst_element_seek(p->pipeline, 1.0, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, (gint64)(p->position*GST_SECOND), GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE);
 }
 
@@ -1209,24 +1315,23 @@ DictoPrivate *p=GET_PRIVATE(d);
 
 g_return_val_if_fail(d, FALSE);
 g_return_val_if_fail(file, FALSE);
-
-if (!p->play.pipeline)
-	return FALSE;
+g_return_val_if_fail(p->play.pipeline, FALSE);
 
 if (p->record.active==TRUE)
 	return FALSE;
 
 p->rforsecs=-1.0;
 
-dicto_set_filename(d, p->play.src, file);
+if (p->state!=DICTO_STATE_PAUSED)
+	dicto_set_filename(d, p->play.src, file);
 
-if (p->play.position>0.0 && dicto_seek_pipeline(&p->play)) {
+if (p->play.doseek && p->play.position>=0.0 && dicto_seek_pipeline(&p->play))
 	g_warning("Seek failed");
-}
 
 if (gst_element_set_state(p->play.pipeline, GST_STATE_PLAYING)==GST_STATE_CHANGE_FAILURE) {
 	g_warning("Failed to play file %s\n", p->cfile);
 	dicto_set_filename(d, p->play.src, NULL);
+	p->state=DICTO_STATE_STOPPED;
 	p->play.active=FALSE;
 	return FALSE;
 }
@@ -1272,7 +1377,7 @@ if (p->tags)
 	g_hash_table_destroy(p->tags);
 p->tags=g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 
-p->play.position=0.0;
+p->play.position=-0.0;
 p->play.length=0.0;
 
 gst_element_set_state(p->play.pipeline, GST_STATE_NULL);
@@ -1327,9 +1432,11 @@ switch (current) {
 	case GST_STATE_VOID_PENDING:
 	case GST_STATE_NULL:
 		p->play.position=0.0;
+		p->play.doseek=TRUE;
 		return FALSE;
 	case GST_STATE_PAUSED:
 		p->play.position=pos;
+		p->play.doseek=TRUE;
 		g_signal_emit(d, signals[SIGNAL_SEEK], 0, pos, NULL);
 		return TRUE;
 	case GST_STATE_READY:
@@ -1339,7 +1446,7 @@ switch (current) {
 }
 
 if (!dicto_seek_pipeline(&p->play)) {
-	g_warning("Seek failed");
+	g_warning("dicto: Seek failed");
 	return FALSE;
 }
 
@@ -1407,7 +1514,7 @@ return TRUE;
 /**
  * dicto_record:
  * @d: A #Dicto object
- * @f: Destination filename to record to.
+ * @file: Destination filename to record to.
  *
  * Records to given file without a limit. Emits #Dicto::recording when recording starts.
  *
@@ -1512,6 +1619,7 @@ GstState current, pending;
 if (p->play.pipeline && p->play.active) {
 	gst_element_get_state(p->play.pipeline, &current, &pending, GST_CLOCK_TIME_NONE);
 	switch (current) {
+	case GST_STATE_VOID_PENDING:
 	case GST_STATE_NULL:
 		return FALSE;
 	case GST_STATE_READY:
@@ -1540,29 +1648,32 @@ gboolean
 dicto_stop(Dicto *d)
 {
 DictoPrivate *p=GET_PRIVATE(d);
-GstState current;
-GstState pending;
 
-if (p->record.pipeline) {
-	gst_element_get_state(p->record.pipeline, &current, &pending, GST_CLOCK_TIME_NONE);
-	if (current==GST_STATE_PLAYING) {
+switch (p->state) {
+	case DICTO_STATE_RECORDING:
+		g_return_val_if_fail(p->record.pipeline, FALSE);
+		g_debug("dicto: Sending EOS");
 		dicto_position_update(d, FALSE);
-		gst_element_send_event(p->record.pipeline, gst_event_new_eos());
+		if (!gst_element_send_event(p->record.pipeline, gst_event_new_eos())) {
+			g_warning("Failed to send EOS, forcing stop");
+			dicto_pipeline_stop(p, &p->record);
+		}
 		return TRUE;
-	}
-}
-
-if (p->play.pipeline) {
-	gst_element_get_state(p->play.pipeline, &current, &pending, GST_CLOCK_TIME_NONE);
-	if (current==GST_STATE_PLAYING || current==GST_STATE_PAUSED) {
+	break;
+	case DICTO_STATE_PAUSED:
+	case DICTO_STATE_PLAYING:
+		g_return_val_if_fail(p->play.pipeline, FALSE);
+		g_debug("dicto: Stopping play");
 		dicto_position_update(d, FALSE);
 		gst_element_set_state(p->play.pipeline, GST_STATE_READY);
 		p->play.active=FALSE;
-		p->rms=0;
+		p->rms=0.0;
 		p->state=DICTO_STATE_STOPPED;
 		g_signal_emit(d, signals[SIGNAL_STOPPED_PLAY], 0, NULL);
 		return TRUE;
-	}
+	break;
+	default:
+		return FALSE;
 }
 
 return FALSE;
@@ -1662,7 +1773,7 @@ dp->length=GST_TIME_TO_SECS(len);
 if (length)
 	*length=dp->length;
 
-if (gst_element_get_state(pipe, &current, &pending, GST_CLOCK_TIME_NONE)==GST_STATE_CHANGE_FAILURE)
+if (gst_element_get_state(pipe, &current, &pending, GST_SECOND)==GST_STATE_CHANGE_FAILURE)
 	return FALSE;
 
 switch (current) {
@@ -1684,6 +1795,20 @@ if (position)
 return r;
 }
 
+/**
+ * dicto_get_tag_setter:
+ *
+ * Returns: The record pipelines #GstTagSetter.
+ */
+GstTagSetter *
+dicto_get_tag_setter(Dicto *d)
+{
+DictoPrivate *p=GET_PRIVATE(d);
+dicto_pipeline *dp;
+dp=&p->record;
+return dp->ts;
+}
+
 /**
  * dicto_set_format:
  * @d: Dicto object
